#!/usr/bin/env node
'use strict';

// Dependencies
const _ = require('lodash');
const util = require('util');
const yargs = require('yargs');
const files = require('../lib/files');
const functions = require('../lib/functions');
const errors = require('../lib/errors');
const logging = require('../lib/logging');
const task = require('../lib/task');
const program = require('../lib/program');
const Promise = require('bluebird');

// Logger
const logger = logging.createLogger('nodus-run');

// Program Help
yargs
    .usage('Usage: $0 <command> [args] [options]')
    .version()
    .epilog('Copyright (c) 2017 Brad Serbu <bradserbu@bradserbu.com>\nAll rights reserved.')
    .help();

/**
 * Display the result of a value to the screen
 * @param value
 */
function display(value) {
    return Promise
        .resolve(value)
        .then(result => console.log(result))
        .catch(err => console.error(err));
}

/**
 * Load a command to run from the target
 * @param target
 * @returns {*}
 */
function loadTask(targetFile) {
    // Load the module/file to be run
    const target = files.requireFile(targetFile);
    logger.debug('TARGET', target);

    const name = target.name || targetFile;
    logger.debug('TARGET_NAME', name);

    const cmd = task.isTask(target)
        ? target
        : task(name, target);

    return cmd;
}

// Parse CommandLine Arguments
const argv = yargs.argv;
logger.debug('ARGV', argv);

// Determine the file/module to be run
const targetFile = argv._.shift();
logger.debug('TARGET_FILE', targetFile);

// Display help if no target is specified
if (util.isNullOrUndefined(targetFile)) {
    return yargs.showHelp();
}

// Load command to run
const cmd = loadTask(targetFile);

// Run the command
const result = cmd.run(argv);
logger.debug('RESULT', result);

// Display the Result
display(result)
    .finally(() => program.shutdown());